package actinScope;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.image.codec.jpeg.*;
import java.text.*;

/* Class CapturingCanvas3D, using the instructions from the Java3D 
    FAQ pages on how to capture a still image in jpeg format.

    A capture button would call a method that looks like


    public static void captureImage(CapturingCanvas3D MyCanvas3D) {
	MyCanvas3D.writeJPEG_ = true;
	MyCanvas3D.repaint();
    }


    Peter Z. Kunszt
    Johns Hopkins University
    Dept of Physics and Astronomy
    Baltimore MD

	<ActinScope - for the simulation of dendritic actin networks>
	Copyright (C) <2005>  <Jonathan B. Alberts>
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

public class CapturingCanvas3D extends Canvas3D  {

    public boolean writeJPEG_;
    public String captureLoc;
    private int postSwapCount_;
    DecimalFormat countFormat = new DecimalFormat ("#0000.#;#0000.#");

    public CapturingCanvas3D(GraphicsConfiguration gc) {
	super(gc);
	postSwapCount_ = 0;
    }

    public void postSwap() {
	if(writeJPEG_) {
	    System.out.println("Writing JPEG");
	    int width = this.getWidth();
	    int height = this.getHeight();
	    GraphicsContext3D  ctx = getGraphicsContext3D();
	    // The raster components need all be set!
	    Raster ras = new Raster(
                   new Point3f(-1.0f,-1.0f,-1.0f),
		   Raster.RASTER_COLOR,
		   0,0,
		   width,height,
		   new ImageComponent2D(
                             ImageComponent.FORMAT_RGB,
			     new BufferedImage(width,height,
					       BufferedImage.TYPE_INT_RGB)),
		   null);

	    ctx.readRaster(ras);

	    // Now strip out the image info
	    BufferedImage img = ras.getImage().getImage();

	    // write that to disk....
	    try {
		FileOutputStream out = new FileOutputStream(captureLoc + File.separator + "file" + String.valueOf(countFormat.format(postSwapCount_))+".jpg");
		JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
		JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(img);
		param.setQuality(1.0f,false); // 100% quality JPEG
		encoder.setJPEGEncodeParam(param);
		encoder.encode(img);
		out.close();
	    } catch ( IOException e ) {
		System.out.println("I/O exception!");
	    }
	    postSwapCount_++;
	    writeJPEG_ = false;
	}
    }
    
    public boolean doneWithCapture () {
    		return !writeJPEG_;
    }
    
    public void setCaptureLoc (String capLoc) {
    		captureLoc = capLoc;
    }
}
