package actinScope;
/*
 * Monomer.java  ... store info about a monomer state and location
 */
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import com.sun.j3d.utils.geometry.*;

import javax.media.j3d.*;
import javax.vecmath.*;

public class Monomer {
	static final int ATP_COLOR = 0;
	static final int ADP_PI_COLOR = 1;
	static final int ADP_COLOR = 2;
	
	static final float actDim = (float)(Env.radOfActin);
	
	Pt3D coord, end1, end2;
	Filament parentFil;
	Monomer plusMon,minusMon;
	
	// graphics
	boolean graphicsMade = false;
	static final Color3f atpC = new Color3f(0.0f,0.0f,1.0f);
	static final Color3f adpPiC = new Color3f(1.0f,0.0f,0.0f);
	static final Color3f adpC = new Color3f(1.0f,1.0f,1.0f);
	static float shiny = 128.0f;
	static final Material atpMat = new Material(atpC,atpC,atpC,atpC,shiny);
	Appearance curApp;
	BranchGroup monG;
	TransformGroup monTG;
	Transform3D monT3D;
	LineArray monLine;
	Sphere monSphere;
	
	public Monomer (Pt3D coord, Filament parentFil) {
		this.coord = new Pt3D(coord);
		this.parentFil = parentFil;
		makeGraphics();
	}
	
	public void sepaku () {
		coord = null;
		end1 = null;
		end2 = null;
		//parentFil = null;
		if (plusMon != null) {
			plusMon.minusMon = null;
			plusMon = null;
		}
		if (minusMon != null) {
			minusMon.plusMon = null;
			minusMon = null;
		}
		monLine = null;
		monSphere = null;
		curApp = null;
		if (monG != null) { monG.detach(); }
		monG = null;
		monTG = null;
		monT3D = null;
	}
	
	public void slideMonomer (double dist, Pt3D uVec) {
		coord.inc(dist,uVec);
		monT3D.setTranslation(new Vector3d(coord.x,coord.y,coord.z));
		monTG.setTransform(monT3D);
	}
	
	public void makeGraphics() {
		curApp = new Appearance();
		curApp.setMaterial(atpMat);
		curApp.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		monLine = new LineArray(2,LineArray.COORDINATES);
		monSphere = new Sphere (actDim,Sphere.GENERATE_NORMALS, Env.lowSphereTessalation,curApp);
		monT3D = new Transform3D();
		monT3D.setTranslation(new Vector3d(coord.x,coord.y,coord.z));
		monTG = new TransformGroup(monT3D);
		monTG.addChild(monSphere);
	
		monG = new BranchGroup();
		monG.setCapability(BranchGroup.ALLOW_DETACH);
		monG.addChild(monTG);
		graphicsMade = true;
	}
	
	public void eraseOldGraphics () {
		try {
			monG.detach();
			graphicsMade = false;
		} catch (java.lang.NullPointerException npe) { }
	}
	
	
	public BranchGroup getGraphics() {
		if (!graphicsMade) {
			makeGraphics();
		}
		return monG;
	}
	
	
	
	
}
