package actinScope;
/*
 * ParamLine... a JLabel, JTextField, with units and assundry methods to parse the input
 */

/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;

import java.text.*;
import java.util.*;

public class ParamGui {
	static final int ALWAYS_ON = 0;
	static final int CHECKBOX_ON = 1;
	int type = ALWAYS_ON;
	Parameter param;
	JLabel paramNameLabel,unitsLabel;
	JTextField paramField;
	JCheckBox paramNameCkBox;
	
	
	// formats
	static int textCols = 7;
	static final int EXPONENTIAL_FORMAT = 1;
	static final int TENSNANOS_FORMAT = 2;
	static final int FRACNANOS_FORMAT = 3;
	static final int INTEGER_FORMAT = 4;
	static DecimalFormat expFormat = new DecimalFormat ("0.00E0");
	static DecimalFormat tensNanosFormat = new DecimalFormat ("0.00");
	static DecimalFormat fracNanosFormat = new DecimalFormat ("0.0000");
	static DecimalFormat intFormat = new DecimalFormat ("0");
	DecimalFormat curFormat = tensNanosFormat;

	public ParamGui (Parameter param) {
		this.type = ALWAYS_ON;
		this.param = param;
		paramNameLabel = new JLabel(param.getName());
		paramNameCkBox = new JCheckBox(param.getName(),true);
		unitsLabel = new JLabel(param.getUnits());
		paramField = new JTextField(textCols);
		refreshField();
		formatAllComponents();
		setDepletable();
	}
	
	public ParamGui (Parameter param, int TYPE) {
		this.type = TYPE;
		this.param = param;
		paramNameLabel = new JLabel(param.getName());
		paramNameCkBox = new JCheckBox(param.getName(),param.depletable);
		unitsLabel = new JLabel(param.getUnits());
		paramField = new JTextField(textCols);
		refreshField();
		formatAllComponents();
		setDepletable();
	}
	
	private void formatAllComponents () {
		paramNameLabel.setHorizontalAlignment(JLabel.CENTER);
		paramNameLabel.setFont(Env.controlFont);
		paramNameLabel.setForeground(Env.controlForeColor);
		
		paramNameCkBox.setHorizontalAlignment(JLabel.LEFT);
		paramNameCkBox.setFont(Env.controlFont);
		paramNameCkBox.setForeground(Env.controlForeColor);
		paramNameCkBox.setBackground(Env.controlBackColor);
		
		paramField.setFont(Env.controlFont);
		paramField.setHorizontalAlignment(JTextField.CENTER);
		paramField.setBackground(Env.controlBackColor);
		paramField.setForeground(Env.controlForeColor);
		paramField.setBorder(BorderFactory.createEmptyBorder());
		
		unitsLabel.setHorizontalAlignment(JLabel.CENTER);
		unitsLabel.setFont(Env.controlFont);
		unitsLabel.setForeground(Env.controlForeColor);
	}
	
	public void addToPanel (JPanel p) {
		if (type == CHECKBOX_ON) {
			p.add(paramNameCkBox);
		} else {
			p.add(paramNameLabel);
		}
		p.add(paramField);
		p.add(unitsLabel);
	}
	
	public void setDepletable () {
		param.setDepletable(paramNameCkBox.isSelected());
	}
		
	public void setNameLabel (String newName) {
		paramNameLabel.setText(newName);
	}
	
	public void refreshField () {
		paramField.setText(curFormat.format(param.getValue()));
	}
	
	public void setUnitsLabel (String newUnits) {
		unitsLabel.setText(newUnits);
	}
	
	public JLabel getNameLabel () {
		return paramNameLabel;
	}
	
	public JTextField getField () {
		return paramField;
	}
	
	public JLabel getUnitsLabel () {
		return unitsLabel;
	}
	
	public double getValue () {
		return param.getValue();
	}
	
	public void setValue (double newVal) {
		param.setValue(newVal);
	}
	
	public void setValueFromGui () {
		String valString = paramField.getText();
		try {
			Double valD = Double.valueOf(valString);
			double val = valD.doubleValue();
			param.setValue(val);
			param.setLastValue(val);	// set lastValue whenever updating from GUI
		} catch (java.lang.NumberFormatException nfe) {
			System.out.println("Invalid field entry in " + param.getName() + "... the change was ignored");
			refreshField();
		}
		setDepletable();
	}
	
	public void setToDefaultValue () {
		param.resetToDefault();
		param.setLastValue(param.getValue());
		refreshField();
	}
	
	public void setToLastValue () {
		param.resetToLastValue();
		refreshField();
	}
	
	
	public void setFieldType (int type) {
		switch (type) {
		case EXPONENTIAL_FORMAT:
			curFormat = expFormat;
			break;
		case TENSNANOS_FORMAT:
			curFormat = tensNanosFormat;
			break;
		case FRACNANOS_FORMAT:
			curFormat = fracNanosFormat;
			break;
		case INTEGER_FORMAT:
			curFormat = intFormat;
			break;
		}
		refreshField();
	}
	
}
