package actinScope;
/* Rdm class.... this class is just storage and creation of a bunch of random numbers.
Create these arrays once at the beginning of a run and then use them throughout
*/
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.io.*;

public class Rdm {
	static int numOfRdmNumbers = 1000000;
	static int rdmNumUse = 5;
	static double deltaT;
	static int arraySize;
	static double [] uniformRnd;
	static UCircRnd [] unitCircleRnd;
	static int dCtr = 0;
	static int cCtr = 0;
	static int uRdmCt = 0;
	static int cRdmCt = 0;
	
	
	public Rdm (double deltaT) {
		this.deltaT = deltaT;
		this.arraySize = numOfRdmNumbers;
		uniformRnd = new double [arraySize];
		unitCircleRnd = new UCircRnd [arraySize];
		mkUniformArray();
		//mkUnitCircleArray();
	}
	
	public static void ckURdmRefresh() {
		if (uRdmCt > rdmNumUse*numOfRdmNumbers) {
			mkUniformArray();
			uRdmCt = 0;
		}
	}
	
	public static void ckCRdmRefresh() {
		if (cRdmCt > rdmNumUse*numOfRdmNumbers) {
			refreshUnitCircleArray();
			cRdmCt = 0;
		}
	}
			
	
	private static void mkUniformArray () {
		//talk ("Creating random number array of size " + String.valueOf(arraySize) + "......... ");
		for (int i = 0; i < arraySize; i++) { uniformRnd[i] = Math.random(); }
		//talkln ("  Done.");
	}
	
	private static void mkUnitCircleArray() {
		//talk ("Creating random pairs in unit circle, size " + String.valueOf(arraySize) + "........");
		for (int i = 0; i < arraySize; i++) {
			unitCircleRnd[i] = new UCircRnd(deltaT);
		}
		//talkln (" Done.");
	}
	
	private static void refreshUnitCircleArray() {
		//talk ("Creating random pairs in unit circle, size " + String.valueOf(arraySize) + "........");
		for (int i = 0; i < arraySize; i++) {
			unitCircleRnd[i].newValue(deltaT);
		}
		//talkln (" Done.");
	}
	
	public static double uRdm() {
		if (dCtr > arraySize -5) { dCtr = 0; ckURdmRefresh(); } else { dCtr ++; }
		uRdmCt++;
		return uniformRnd[dCtr];
	}
	
	public static UCircRnd cRdm() {
		if (cCtr > arraySize -5) { cCtr = 0; ckCRdmRefresh(); } else { cCtr ++; }
		cRdmCt++;
		return unitCircleRnd [cCtr];
	}

	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.print(info);
	}
}
