package actinScope;
/* 
RunControlFrame... basic run controls and info on concentrations and stats
*/
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.media.j3d.*;

public class RunControlFrame extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	ActinScope parentSim;
	static Cell theCell;
	JPanel titlePanel;
	JPanel continuePanel;
	JPanel plotOptPanel;
	BareButton goB,pauseB,redoB;
	JLabel timeLabel;
	JTextField timeField, filNumField, branchStatsField, filStatsField, filAngPlusStatsField,filAngMinusStatsField;
	JCheckBox branchStatsBox, filStatsBox, filAngPlusStatsBox, filAngMinusStatsBox;  
	boolean fieldError = false;
	

	
	// text formats
	int decPtCt = 2;
	DecimalFormat expFormat = new DecimalFormat ("0.00E0");
	DecimalFormat decFormat = new DecimalFormat ("0.00");
	DecimalFormat intFormat = new DecimalFormat ("0");
	DecimalFormat timeFormat = new DecimalFormat ("0.0000");
	DecimalFormat branchFormat = new DecimalFormat ("0.000");
	DecimalFormat textFormat = expFormat;

	public RunControlFrame (ActinScope parentSim) {
		this.parentSim = parentSim;
		this.setBackground(Env.controlBackColor);
		this.setResizable(false);
		
		makeTitlePanel();
		makeContinuePanel();
		makePlotOptPanel();
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		initialize();
		Dimension prefDim = getPreferredSize();
		prefDim.height += 40;
		setSize(prefDim);
	}
	
	public void initialize () {
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (titlePanel, BorderLayout.NORTH);
		getContentPane().add (plotOptPanel, BorderLayout.CENTER);
		getContentPane().add (continuePanel, BorderLayout.SOUTH);
	}
	
	public void setCell (Cell theCell) {
		this.theCell = theCell;
	}
			
	public void setInfoFields () {
		timeField.setText(timeFormat.format(parentSim.simulationTime));
		filNumField.setText(intFormat.format(Filament.filamentCt));
		
		if (branchStatsBox.isSelected()) {
			Stat branchStats = Filament.branchStatistics();
			String bStatStr = branchFormat.format(branchStats.getMean()) + " +/- " + branchFormat.format(branchStats.getStdDev());
			branchStatsField.setText(bStatStr);
		}
		
		if (filStatsBox.isSelected()) {
			Stat filStats = Filament.filLengthStatistics();
			String fStatStr = branchFormat.format(filStats.getMean()) + " +/- " + branchFormat.format(filStats.getStdDev());
			filStatsField.setText(fStatStr);
		}
		
		if (filAngPlusStatsBox.isSelected()) {
			Stat filAngStats = Filament.filAnglePlusStatistics();
			String fStatStr = branchFormat.format(filAngStats.getMean()) + " +/- " + branchFormat.format(filAngStats.getStdDev());
			filAngPlusStatsField.setText(fStatStr);
		}
		
		if (filAngMinusStatsBox.isSelected()) {
			Stat filAngMinusStats = Filament.filAngleMinusStatistics();
			String fStatStr = branchFormat.format(filAngMinusStats.getMean()) + " +/- " + branchFormat.format(filAngMinusStats.getStdDev());
			filAngMinusStatsField.setText(fStatStr);
		}
		
		
		//if (!this.isVisible()) {setVisible(true); }
	}
	
	public void setPaused () {
		goB.setEnabled(true);
		pauseB.setEnabled(false);
	}
	
	public void setRunning () {
		goB.setEnabled(false);
		pauseB.setEnabled(true);
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Redo")) {
			parentSim.setPaused();
			parentSim.redoRun();
		}
		
		if (act.equals("Run")) {
			parentSim.setRunning();
		}
		
		if (act.equals("Pause")) {
			parentSim.setPaused();
		}
		
	}
	
	public void stateChanged(ChangeEvent e) {
	    JSlider source = (JSlider)e.getSource();
	}
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	public void makeTitlePanel () {
		titlePanel = new JPanel();
		titlePanel.setBackground(Env.controlBackColor);
		JLabel titleLabel = new JLabel("Run Statistics");
		titleLabel.setBackground(Env.controlBackColor);
		titleLabel.setForeground(Env.controlForeColor);
		titlePanel.add(titleLabel);
	}
	
	public void makePlotOptPanel() {
		plotOptPanel = new JPanel();
		plotOptPanel.setBackground(Env.controlBackColor);
		
		JLabel dummyLabel = new JLabel("");
		
		JLabel plotTypeLabel = new JLabel("Plot Type:");
		plotTypeLabel.setFont(Env.controlFont);
		
		// text field dimension
		Dimension textDim = new Dimension(50,20);
		
		timeLabel = new JLabel(" Simulation Time: ");
		timeLabel.setFont(Env.controlFont);
		timeLabel.setForeground(Env.controlForeColor);
	
		timeField = new JTextField(6);
		timeField.setFont(Env.controlFont);
		timeField.setHorizontalAlignment(JTextField.CENTER);
		timeField.setEditable(false);
		timeField.setPreferredSize(textDim);
		timeField.setBackground(Env.controlBackColor);
		timeField.setForeground(Env.controlForeColor);
		timeField.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel timeUnits = new JLabel (" seconds");
		timeUnits.setFont(Env.controlFont);
		timeUnits.setForeground(Env.controlForeColor);
		
		JLabel filNumLabel = new JLabel(" Filament Number: ");
		filNumLabel.setFont(Env.controlFont);
		filNumLabel.setForeground(Env.controlForeColor);
	
		filNumField = new JTextField(6);
		filNumField.setFont(Env.controlFont);
		filNumField.setHorizontalAlignment(JTextField.CENTER);
		filNumField.setEditable(false);
		filNumField.setPreferredSize(textDim);
		filNumField.setBackground(Env.controlBackColor);
		filNumField.setForeground(Env.controlForeColor);
		filNumField.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel filNumUnits = new JLabel (" ");
		filNumUnits.setFont(Env.controlFont);
		filNumUnits.setForeground(Env.controlForeColor);
		
		branchStatsBox = new JCheckBox (" Branch Distance: ");
		branchStatsBox.setHorizontalAlignment(JLabel.LEFT);
		branchStatsBox.setFont(Env.controlFont);
		branchStatsBox.setForeground(Env.controlForeColor);
		branchStatsBox.setBackground(Env.controlBackColor);
		branchStatsBox.setSelected(true);
		
		branchStatsField = new JTextField(6);
		branchStatsField.setFont(Env.controlFont);
		branchStatsField.setHorizontalAlignment(JTextField.CENTER);
		branchStatsField.setEditable(false);
		branchStatsField.setPreferredSize(textDim);
		branchStatsField.setBackground(Env.controlBackColor);
		branchStatsField.setForeground(Env.controlForeColor);
		branchStatsField.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel branchStatsUnits = new JLabel (" microns");
		branchStatsUnits.setFont(Env.controlFont);
		branchStatsUnits.setForeground(Env.controlForeColor);
		
		filStatsBox = new JCheckBox (" Filament Length: ");
		filStatsBox.setHorizontalAlignment(JLabel.LEFT);
		filStatsBox.setFont(Env.controlFont);
		filStatsBox.setForeground(Env.controlForeColor);
		filStatsBox.setBackground(Env.controlBackColor);
		filStatsBox.setSelected(true);
		
		filStatsField = new JTextField(6);
		filStatsField.setFont(Env.controlFont);
		filStatsField.setHorizontalAlignment(JTextField.CENTER);
		filStatsField.setEditable(false);
		filStatsField.setPreferredSize(textDim);
		filStatsField.setBackground(Env.controlBackColor);
		filStatsField.setForeground(Env.controlForeColor);
		filStatsField.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel filStatsUnits = new JLabel (" microns");
		filStatsUnits.setFont(Env.controlFont);
		filStatsUnits.setForeground(Env.controlForeColor);
		
		filAngPlusStatsBox = new JCheckBox (" Forward-facing filament angle: ");
		filAngPlusStatsBox.setHorizontalAlignment(JLabel.LEFT);
		filAngPlusStatsBox.setFont(Env.controlFont);
		filAngPlusStatsBox.setForeground(Env.controlForeColor);
		filAngPlusStatsBox.setBackground(Env.controlBackColor);
		filAngPlusStatsBox.setSelected(true);
		
		filAngPlusStatsField = new JTextField(6);
		filAngPlusStatsField.setFont(Env.controlFont);
		filAngPlusStatsField.setHorizontalAlignment(JTextField.CENTER);
		filAngPlusStatsField.setEditable(false);
		filAngPlusStatsField.setPreferredSize(textDim);
		filAngPlusStatsField.setBackground(Env.controlBackColor);
		filAngPlusStatsField.setForeground(Env.controlForeColor);
		filAngPlusStatsField.setBorder(BorderFactory.createEmptyBorder());
		
		filAngMinusStatsBox = new JCheckBox (" Back-facing filament angle: ");
		filAngMinusStatsBox.setHorizontalAlignment(JLabel.LEFT);
		filAngMinusStatsBox.setFont(Env.controlFont);
		filAngMinusStatsBox.setForeground(Env.controlForeColor);
		filAngMinusStatsBox.setBackground(Env.controlBackColor);
		filAngMinusStatsBox.setSelected(true);
		
		filAngMinusStatsField = new JTextField(6);
		filAngMinusStatsField.setFont(Env.controlFont);
		filAngMinusStatsField.setHorizontalAlignment(JTextField.CENTER);
		filAngMinusStatsField.setEditable(false);
		filAngMinusStatsField.setPreferredSize(textDim);
		filAngMinusStatsField.setBackground(Env.controlBackColor);
		filAngMinusStatsField.setForeground(Env.controlForeColor);
		filAngMinusStatsField.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel filAngStatsUnits = new JLabel (" degrees");
		filAngStatsUnits.setFont(Env.controlFont);
		filAngStatsUnits.setForeground(Env.controlForeColor);
		
		JLabel filAngStatsUnits2 = new JLabel (" degrees");
		filAngStatsUnits2.setFont(Env.controlFont);
		filAngStatsUnits2.setForeground(Env.controlForeColor);
		
		plotOptPanel.setLayout(new GridLayout(6,3));
		plotOptPanel.add(timeLabel);
		plotOptPanel.add(timeField);
		plotOptPanel.add(timeUnits);
		plotOptPanel.add(filNumLabel);
		plotOptPanel.add(filNumField);
		plotOptPanel.add(filNumUnits);
		plotOptPanel.add(branchStatsBox);
		plotOptPanel.add(branchStatsField);
		plotOptPanel.add(branchStatsUnits);
		plotOptPanel.add(filStatsBox);
		plotOptPanel.add(filStatsField);
		plotOptPanel.add(filStatsUnits);
		plotOptPanel.add(filAngPlusStatsBox);
		plotOptPanel.add(filAngPlusStatsField);
		plotOptPanel.add(filAngStatsUnits);
		plotOptPanel.add(filAngMinusStatsBox);
		plotOptPanel.add(filAngMinusStatsField);
		plotOptPanel.add(filAngStatsUnits2);
	}
	
	public void makeContinuePanel () {
		continuePanel = new JPanel();
		continuePanel.setBackground(Env.controlBackColor);
	
		goB = new BareButton("Run");
		goB.setActionCommand("Run");
		goB.setToolTipText("Start run");
		goB.addActionListener(this);
		goB.setEnabled(true);
		
		redoB = new BareButton ("Redo");
		redoB.setActionCommand("Redo");
		redoB.setToolTipText("Redo Run");
		redoB.addActionListener(this);
		redoB.setEnabled(true);
		
		pauseB = new BareButton("Pause");
		pauseB.setActionCommand("Pause");
		pauseB.setToolTipText("Pause run");
		pauseB.addActionListener(this);
		pauseB.setEnabled(false);
		
		continuePanel.setLayout(new FlowLayout(FlowLayout.CENTER,10,10));
		continuePanel.add(redoB);
		continuePanel.add(pauseB);
		continuePanel.add(goB);
		
		
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	
	
}