package actinScope;
/*
	// Thing.... the superclass for all moving objects in this demonstration
*/
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Thing extends Object {
	// some physical contants/parameters
	static final double Boltz = 1.380662e-23;			// Boltzman's constant in J/K
	static final double tempK = 298.15;	// 25 deg Celsius in Kelvin
	static final double aeta = 0.1;		// viscosity in bug.... Pa-s (1e-3 is water)
	static double AvogadroNum = 6.02214199e23;			// Avogadro's number.. #/Mole
	
	static Thing [] theThings = new Thing [100000];		// array of all things
	static int thingCt = 0;								// how many things
	static double deltaT = ActinScope.deltaT;
	static Cell theCell;					// only one bug
	// for efficiency
	static final Pt3D xUnitVector = new Pt3D(1,0,0);	// unit vector along body-fixed x-axis
	static final Pt3D yUnitVector = new Pt3D(0,1,0);	// unit vector along body-fixed y-axis
	static final Pt3D zUnitVector = new Pt3D(0,0,1);	// unit vector along body-fixed z-axis
	static final Pt3D zeroVec = new Pt3D();	// just a zero Pt3D
	int myThingNumber;					// identifies where in "theThings" this Thing is
	boolean removeMe = false;			// if true this Thing will be eliminated
	Pt3D coord = new Pt3D();			// the x,y,and z position of the Thing
	Pt3D angle = new Pt3D();			//
	static Pt3D maxPos = ActinScope.worldDimension;	// maximum x position this Thing can occupy
	double [][] transXTox = new double [3][3];	// transformation matrix from fixed to body-fixed frame
	double [][] transxToX = new double [3][3];	// the inverse transformation... body-fixed to fixed
	Matrix3d mxToX = new Matrix3d();
	Matrix3d mXTox = new Matrix3d();
	Pt3D uVec = new Pt3D();		// the unit vector that describes the orientation of the player
	Pt3D t1Vec = new Pt3D();		// the first transvers vector for this body... in y direction
	Pt3D t2Vec = new Pt3D();		// the second tranverse vector.. in z direction
	Pt3D cosAng = new Pt3D();	// cosines of Euler angles {psi,theta,phi}
	Pt3D sinAng = new Pt3D();	// sines of Euler angles {psi,theta,phi}
	
	// some Pt3Ds used in calculating random forces
	Pt3D v1 = new Pt3D();
	Pt3D v2 = new Pt3D();
	Pt3D rsq = new Pt3D();
	Pt3D facterm =new Pt3D();
	Pt3D fac1 = new Pt3D();
	Pt3D fac2 = new Pt3D();
	Pt3D tempPt = new Pt3D();
	
	// for graphics
	double gScale = 1; // scale up/down for converting physical dim to graphics dim
	boolean inGroup = false;
	boolean graphicsMade = false;
	BranchGroup G = new BranchGroup();
	TransformGroup g3d = new TransformGroup();
	Transform3D t3d = new Transform3D();
	Appearance a = new Appearance();
	Material m;
	static final Vector3d farFarAway = new Vector3d(1000,1000,1000);
	static final Point3d farfarAwayPt = new Point3d(1000,1000,1000);
	
	public Thing (Pt3D initCoord) {
		this.coord.copy(initCoord);
		addThing(this);
	}
	
	public Thing (Pt3D initCoord, Pt3D initAngle) {
		this.coord.copy(initCoord);
		this.angle.copy(initAngle);
		addThing(this);
	}
	public void initialize(){}
	public void calculateProperties() {}
	public void moveThing() {}
	public void step () {
		// put the code here to move this object each time-step
	}
	
	public void drawYourself (Graphics g, double scale, double [] offset) {
		// put the code here to draw the object on "g"
	}
	
	public void divide() {}
	
	public void transMat () {
		transXTox [0][0] = cosAng.x*cosAng.y;
		transXTox [0][1] = sinAng.x*cosAng.y;
		transXTox [0][2] = -sinAng.y;
		transXTox [1][0] = -sinAng.x*cosAng.z+cosAng.x*sinAng.y*sinAng.z;
		transXTox [1][1] = cosAng.x*cosAng.z+sinAng.x*sinAng.y*sinAng.z;
		transXTox [1][2] = cosAng.y*sinAng.z;
		transXTox [2][0] = sinAng.x*sinAng.z+cosAng.x*sinAng.y*cosAng.z;
		transXTox [2][1] = -cosAng.x*sinAng.z+sinAng.x*sinAng.y*cosAng.z;
		transXTox [2][2] = cosAng.y*cosAng.z;
		// the inverse transformation is found as the transpose of this matrix
		// since the direction cosine matrix is orthogonal.
		double curVal;
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				curVal = transXTox [j][i];
				transxToX [i][j] = curVal;
				// set Matrix3d versions
				mxToX.setElement(i,j,curVal);
				mXTox.setElement(j,i,curVal);
			}
		}
	}
	public void resetCounters() {
	}
	
	public double getRdmDelta (){
		return Rdm.uRdm()*2-1;
	}
	
	public static void addThing (Thing newThing) {
		newThing.myThingNumber = thingCt;
		theThings[thingCt] = newThing;
		thingCt++;
	}
	
	public static void removeThing (Thing deadThing) {
		int swapId = deadThing.myThingNumber;
		theThings[swapId] = theThings[thingCt-1];
		theThings[swapId].myThingNumber = swapId;
		sepakuThing(deadThing);
		thingCt += -1;
	}
	
	protected void sepaku () {
		coord = null;
		angle = null;
		transXTox = null;
		transxToX = null;
		mxToX = null;
		mXTox = null;
		uVec = null;
		t1Vec = null;
		t2Vec = null;
		cosAng = null;
		sinAng = null;
		
		// some Pt3Ds used in calculating random forces
		v1 = null;
		v2 = null;;
		rsq = null;;
		facterm =null;;
		fac1 = null;;
		fac2 = null;;
		tempPt = null;;
		
		// for graphics
		G = null;
		g3d = null;
		t3d = null;
		a = null;
		m = null;
	}
	
	public static void sepakuThing (Thing deadThing) {
		
		deadThing = null;
	}
		
	public static void removeDeadThings () {
		for (int i=0;i<thingCt;i++) {
			if (theThings[i] == null) { break; }		// this means we've gotten to the end of our shortening list of things
			if (theThings[i].removeMe) {
				if (theThings[i].graphicsMade) { theThings[i].G.detach(); }
				removeThing(theThings[i]);
			}
		}
	}

	public static Vector3d getGraphicsLoc (Pt3D coord) {
		Pt3D toBugCoord = Pt3D.Sub(coord,theCell.coord);
		//toBugCoord.scale(1/ParMSpindle.worldScale);
		return (new Vector3d(toBugCoord.x,toBugCoord.y,toBugCoord.z));
	}
	public void setGraphicsCapabilities () {
		g3d.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		g3d.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		
		G.setCapability(BranchGroup.ALLOW_DETACH);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		
		a.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		
	}
	
	public void makeGraphics () {}
	public void updateGraphics () {}
	public void advanceGraphics (BranchGroup baseBG) {
		if (!graphicsMade) { 
			makeGraphics(); 
			updateGraphics();
			baseBG.addChild(G);
		} else {
			updateGraphics();
		}
	}

	public Node getGraphicsNode () {
		if (!graphicsMade) { makeGraphics();}
		//inGroup = true;
		updateGraphics();
		return G;
	}
	public void detachGraphics () {
		G.detach();
	}
	public static void talk (String info) {
		System.out.print(info);
	}
	
	public static void talkln (String info) {
		System.out.println(info);
	}
}

